package org.cocos2dx.cpp;

import org.cocos2dx.cpp.BaseActivity;
import org.cocos2dx.cpp.BluetoothHolder;

import java.util.ArrayList;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Vibrator;
import android.util.Log;
import android.view.WindowManager;

public class AppActivity extends BaseActivity
{
	private static AppActivity instance = null;
	
	private static BluetoothHolder bluetoothHolder;
	private static ArrayList<BluetoothDevice> devices;
	
	private static final String TAG = "AppActivity";
    private static final boolean D = false;
      
    private static byte[] reciveMsg2 = new byte[1024];
    private static ArrayList<Byte> MSG = new ArrayList<Byte>();
    private static int msgLenght = 0;

    private static boolean isRecieving = false;   // 是否正在接收消息
    private static boolean isReading = false;
    
    private static boolean isDestroy = false;
    private static String  _DevicName = "";   // 扫描到的设备名称
    private static boolean isSupportBLE = false;
    
    public native static void ReadMsg(byte[] data);  // c++读数据接口函数
    

 
    @Override
    protected void onCreate(Bundle savedInstanceState) 
    {
    	Log.d(TAG, "onCreate");
        super.onCreate(savedInstanceState);
        
        // 全屏显示 且显示状态栏
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_LAYOUT_IN_SCREEN);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS);
        
        // 不休眠
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        
        instance = this;
        devices = new ArrayList<BluetoothDevice>();
        isSupportBLE = BluetoothHolder.init(this);
        
        for(int i=0; i<reciveMsg2.length; i++)
        {
        	reciveMsg2[i]=0;
        }
        MSG.clear();
    }
    
    @Override
    protected void onResume() {
    	Log.d(TAG, "onResume");
        super.onResume();
        bluetoothResume();
    }

    @Override
    protected void onPause() {
    	Log.d(TAG, "onPause");
        super.onPause();
        bluetoothDestroy(false);
    }

    @Override
    protected void onDestroy() {
    	Log.d(TAG, "onDestroy");
        super.onDestroy();
        bluetoothDestroy(true);
    }
    
    private void bluetoothResume()
    {
        bluetoothHolder = BluetoothHolder.getInstance();
        isSupportBLE = bluetoothHolder.init(this);
        bluetoothHolder.openBluetooth();
        bluetoothHolder.bindService();
        bluetoothHolder.registerReceiver(this);
        isDestroy = false;
        Log.d(TAG, "buletoothResume");
    }
    
    private void bluetoothDestroy(boolean clearDevice)
    {
        Disconnect();
        // 解除绑定
        bluetoothHolder.unbindService(clearDevice);
        bluetoothHolder.unregisterReceiver(clearDevice);
        isDestroy = true;
    }
 
    private final BluetoothAdapter.LeScanCallback leScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {
        	if(device.getName() == null) return;
            //Log.d("onLeScan", device.getName() + "--" + rssi);
            if (device.getName().startsWith("M6") || device.getName().startsWith("Mini") || device.getName().startsWith("A6")) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        for (BluetoothDevice d : devices) {
                            if (d.getName().equals(device.getName())) {
                                return;
                            }
                        }
                        devices.add(device);
                        _DevicName += device.getName() + "*";
                        //if (D)  Log.d(TAG, "add device name:"+device.getName());
                    }
                });
           }
        }
    };
    
    // 断开连接回调
    @Override
    public void disconnected(Intent intent) {
        Log.e(TAG, "disconnected");
    }

    // 连接成功回调
    @Override
    public void connected(Intent intent) {
        Log.e(TAG, "connected------");
    }

    // 发现服务
    @Override
    public void discovered(Intent intent) {
        Log.d(TAG, "discovered");
        BluetoothHolder.getInstance().enableTXNotification();
    }
 
    // 读数据
    public synchronized void data(Intent intent, byte[] data) 
    {
        if (data != null)
        {
        	for(int i=0; i<data.length; i++)
        	{
        		//Log.d("data","data"+i+"="+data[i]);
        		MSG.add(data[i]);
        	}	
        	
        	// 调用C++接口，把数据传过去
        	ReadMsg(data);
        }
    }
     
    // 获取实例
    public static Object GetInstance()
    {
    	return instance;
    }
    
    // 初始化
    public boolean initBluetooth()
    {
    	if(!isSupportBLE) isSupportBLE = BluetoothHolder.init(this);
    	Log.e(TAG, "isSupportBLE:"+isSupportBLE);
        return isSupportBLE;
    }
    
    // 开始扫描蓝牙设备
    public void BeginScanBLE()
    {
        if (D) Log.d(TAG, "BeginScanBLE()");
    	devices.clear();
    	// 开始扫描时有连接的话先要断开
    	_DevicName = "";
    	Disconnect();
        bluetoothDestroy(true);
        bluetoothResume();
        BluetoothHolder.getInstance().scanDevice(true, leScanCallback);
    }
    
    // 结束扫描蓝牙设备
    public void FinshScanBLE()
    {
    	if (D) Log.d(TAG, "FinshScanBLE()");
    	//devices.clear();
        BluetoothHolder.getInstance().scanDevice(false, leScanCallback);
    }
    
    // 获取扫描到的蓝牙设备
    public static String GetScanBleList()
    {
    	return _DevicName;   
    }
    
    // 连接指定设备 
    public void ConnectBLE(int deviceId)
    {
    	if (D) Log.d(TAG, "ConnectBLE:"+deviceId);
    	FinshScanBLE();
    	BluetoothDevice device = devices.get(deviceId);
    	BluetoothHolder.getInstance().connect(device.getName(), device.getAddress());
    }
    
    // 断开连接
    public void Disconnect()
    {
    	if(IsConnected())
    	{
    		BluetoothHolder.getInstance().disconnect();
    	}
    	MSG.clear();
    }
    
    // 是否有连接设备
    public boolean IsConnected()
    {
    	return BluetoothHolder.getInstance().isConnected();
    }
    
    // 发送消息
    public void SendMessage(byte[] buf, int len)
    {
    	if (D) Log.d(TAG, "SendMessage len = " + len);
    	if(IsConnected())
    	{
	    	byte[] send = new byte[len];
	        for (int i = 0; i < len; i++) {
	        		send[i] = buf[i];  
	        		if (D) Log.d(TAG, "SendMessage["+i+"] = "+buf[i]);
	        }
        	BluetoothHolder.getInstance().writeCharacteristic(send);
        }
    }
    
    /*** @param time 震动时间*/
    public static void vibrate(long time) {
    Vibrator v = (Vibrator) instance.getSystemService(Context.VIBRATOR_SERVICE);
    v.vibrate(time);
    }

    /** * @param pattern 震动时间数组 EG:{500,200,500,300}*@param repeat 重复次数*/
    public static void vibrateWithPattern(long[] pattern, int repeat) {
    Vibrator v = (Vibrator) instance.getSystemService(Context.VIBRATOR_SERVICE);
    v.vibrate(pattern,repeat);
    }

    /*** 取消震动 */
    public static void cancelVibrate() {
    Vibrator v = (Vibrator) instance.getSystemService(Context.VIBRATOR_SERVICE);
    v.cancel();
    }
    
    public void MyResume() 
    {

    }

    public void MyPause()
    {
         Disconnect();
    }
}

