
package org.cocos2dx.cpp;

import org.cocos2dx.cpp.BluetoothLeService;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.os.IBinder;
import android.text.TextUtils;
import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;


/**
 * @说 明: 蓝牙管理者
 */
public class BluetoothHolder{// extends Activity{
	//private static BluetoothHolder instance = null;
    private static final String TAG = BluetoothHolder.class.getSimpleName();
    public static final UUID UUID_NOTIFY2 = UUID.fromString("6E400002-B5A3-F393-E0A9-E50E24DCCA9E");
    //public static final UUID UUID_Read = UUID.fromString("6E400003-B5A3-F393-E0A9-E50E24DCCA9E");
    public static final UUID UUID_SERVICE2 = UUID.fromString("6E400001-B5A3-F393-E0A9-E50E24DCCA9E");//00001800-0000-1000-8000-00805f9b34fb");
    public static final UUID UUID_NOTIFY = UUID.fromString("0000ffe1-0000-1000-8000-00805f9b34fb");
    public static final UUID UUID_SERVICE = UUID.fromString("0000ffe0-0000-1000-8000-00805f9b34fb");

    private static BluetoothHolder holder = new BluetoothHolder();
    private Context context;
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothManager bluetoothManager;
    private BluetoothLeService bluetoothLeService;
    private BluetoothGattCharacteristic notifyCharacteristic;
    private BroadcastCallback broadcastCallback;
    
    private static int _isSmallType = 1;
    
    /**
     * 扫描标志
     */
    private boolean scanning;
    /**
     * 设备名称
     */
    private String deviceName;
    /**
     * 设备MAC地址
     */
    private String deviceAddress;
    
    /**
     * 获取连接的设备名称
     *
     * @return
     */
    public String getDeviceName() {
        return deviceName;
    }

    /**
     * 获取连接的设备地址
     *
     * @return
     */
    public String getDeviceAddress() {
        return deviceAddress;
    }

    /**
     * 可用发服务与特征集合
     */
    private HashMap<BluetoothGattService,
            ArrayList<BluetoothGattCharacteristic>> gattServiceData;
    private ArrayList<BluetoothGattCharacteristic> characteristics = new ArrayList<BluetoothGattCharacteristic>();

    private BluetoothHolder() {
    }

    /**
     * 初始化蓝牙管理者
     *
     * @param context
     * @return false:初始化失败或者该设备不支持BLE功能
     */

    public static boolean init(Context context) {
        holder.context = context;
        boolean b = holder.initBluetootAdaper();
        return b;
    }

    /**
     * 获取蓝牙管理者
     *
     * @return
     */
    public static BluetoothHolder getInstance() {
        if (holder.context == null) {
            throw new NullPointerException("请在此之前先初始化：BluetoothHolder.init(context)");
        }
        return holder;
    }

    /**
     * 判断是否支持BLE
     *
     * @return true:设备支持BLE功能,否则不支持。
     */
    public boolean isSupportedBLE() {
        return context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE);
    }

    /**
     * 初始化蓝牙adapter.
     *
     * @return false:初始化失败或者该设备不支持BLE功能
     */
    //private boolean initBluetootAdaper() {
    public boolean initBluetootAdaper() {
        if (isSupportedBLE()) {
            bluetoothManager = (BluetoothManager) context.getSystemService(Context.BLUETOOTH_SERVICE);
            bluetoothAdapter = bluetoothManager.getAdapter();
            if(bluetoothAdapter == null) Log.e(TAG,"bluetoothAdapter == null");
            return true;
        } else 
        {
        	Log.d(TAG, "不支持蓝牙");
            return false;
        }
    }

    /**
     * 打开蓝牙
     *
     * @return false:打开失败或已打开
     */
    public boolean openBluetooth() {
        if (bluetoothAdapter != null && !isOpen()) {
        	Log.d(TAG, "打开蓝牙");
            return bluetoothAdapter.enable();
        }
        //Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        //startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        return false;
    }

    /**
     * 是否已经打开蓝牙
     *
     * @return false:未打开
     */
    public boolean isOpen() {
        if (bluetoothAdapter != null) {
            return bluetoothAdapter.isEnabled();
        }
        return false;
    }

    /**
     * 扫描蓝牙设备
     *
     * @param scan     true:开启扫描，否则停止扫描
     * @param callback 扫描回调
     */
    public void scanDevice(boolean scan, BluetoothAdapter.LeScanCallback callback) {
        if (bluetoothAdapter != null) {
            scanning = scan;
            if (scanning) {
            	Log.d(TAG, "开始扫描蓝牙");
                bluetoothAdapter.startLeScan(callback);
            } else {
            	Log.d(TAG, "结束扫描蓝牙");
                bluetoothAdapter.stopLeScan(callback);
            }
        }
        else  Log.d(TAG, "bluetoothAdapter == null");
    }

    /**
     * 是否在扫描
     *
     * @return
     */
    public boolean isScanning() {
        return scanning;
    }

    private boolean bindService;

    public boolean isBindService() {
        return bindService;
    }

    /**
     * 绑定服务
     */
    public void bindService() {
        if (!bindService) {
	    	Log.d(TAG, "绑定服务");
	        Intent gattServiceIntent = new Intent(context, BluetoothLeService.class);
	        bindService = context.bindService(gattServiceIntent, serviceConnection, Context.BIND_AUTO_CREATE);
	        //bindService = context.bindService(gattServiceIntent, serviceConnection, Context.BIND_AUTO_CREATE);
	        if(bindService) Log.d(TAG, "绑定成功");
	        else Log.e(TAG, "绑定失败");
        }
    }

    /**
     * 解绑服务
     */
    public void unbindService(boolean clearDevice) {
        if (bindService) {
        	Log.d(TAG, "解除绑定服务");
            context.unbindService(serviceConnection);
        }
        bluetoothLeService.close();
        bindService = false;
        bluetoothLeService = null;
    }

    /**
     * 连接到蓝牙设备服务
     *
     * @param address 目标设备的设备地址
     * @param name    目标设备的设备名称
     * @return true：连接成功了。通过异步连接结果报告
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public boolean connect(String name, String address) {
        Log.e(TAG, "connect");
        if (bluetoothLeService != null) {
            deviceName = name;
            deviceAddress = address;
//            sharedPreferences.edit().putString("deviceAddress", deviceAddress).commit();
            return bluetoothLeService.connect(deviceAddress);
        }
        Log.e(TAG, "bluetoothLeService == null");
        return false;
    }

    /**
     * 获取连接状态
     *
     * @return BluetoothLeService.STATE_DISCONNECTED:断开状态 <p>
     * BluetoothLeService.STATE_CONNECTING:连通状态 <p>
     * BluetoothLeService.STATE_CONNECTED :保持通话状态 <p>
     */
//    public int getConnectionState() {
//        if (bluetoothLeService != null) {
//            return bluetoothLeService.getConnectionState();
//        }
//        return BluetoothLeService.STATE_DISCONNECTED;
//    }

    /**
     * 取消或断开一个现有的连接，通过异步分离结果报告
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public void disconnect() {
        characteristics.clear();
        if (bluetoothLeService != null) {
        	Log.d(TAG, "断开连接");
            bluetoothLeService.disconnect();
        }
    }

    /**
     * 读取蓝牙设备输入的信息
     */
    public void readCharacteristic() {
        if (gattServiceData != null && !gattServiceData.isEmpty()) {
            for (ArrayList<BluetoothGattCharacteristic> characteristics : gattServiceData.values()) {
                if (characteristics == null) return;
                for (BluetoothGattCharacteristic characteristic : characteristics) {
                    int flage = characteristic.getProperties();
                    if ((flage | BluetoothGattCharacteristic.PROPERTY_READ) > 0) {
                        if (notifyCharacteristic != null) {
                            bluetoothLeService.setCharacteristicNotification(notifyCharacteristic, false,_isSmallType);
                            notifyCharacteristic = null;
                        }
                        bluetoothLeService.readCharacteristic(characteristic);
                    }
                    if ((flage | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
                        notifyCharacteristic = characteristic;
                        bluetoothLeService.setCharacteristicNotification(notifyCharacteristic, true,_isSmallType);
                    }
                }
            }
        }
    }
    
    // 小车收信息必须要用
    public void enableTXNotification()
    {
    	if(_isSmallType == 0) return;
    	if(bluetoothLeService != null){
    		bluetoothLeService.enableTXNotification();
    	}
    }
    

    /**
     * 指定设备发送数据
     *
     * @param writeBytes 16进制的指令
     */
    public void writeCharacteristic(byte[] writeBytes) {
        if (bluetoothLeService == null) {
        	Log.e(TAG,"bluetoothLeService == null");
            return;
        }
        if(_isSmallType == 1) 
        {
        	bluetoothLeService.writeCharacteristic(writeBytes);
        }
        else
        {
            if (gattServiceData != null && !gattServiceData.isEmpty()) {
                for (ArrayList<BluetoothGattCharacteristic> characteristics : gattServiceData.values()) {
                    for (BluetoothGattCharacteristic characteristic : characteristics) {
                        final int charaProp = characteristic.getProperties();
                        notifyCharacteristic = null;
                        if ((charaProp | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
                            notifyCharacteristic = characteristic;
                            if (bluetoothLeService != null)
                            {
                                bluetoothLeService.setCharacteristicNotification(characteristic, true,_isSmallType);
                            }
                        }
                        // 如果该char可写
                        if ((charaProp | BluetoothGattCharacteristic.PROPERTY_WRITE) > 0) {
                            //byte[] value = new byte[1024];
                            //value[0] = (byte) 0x00;
                            //characteristic.setValue(value[0], BluetoothGattCharacteristic.FORMAT_UINT8, 0);
                            //characteristic.setValue(writeBytes);
                        	characteristic.setValue(writeBytes);
                            if (notifyCharacteristic != null)
                            {
                            	//bluetoothLeService.setCharacteristicNotification(notifyCharacteristic,false);
                                //notifyCharacteristic = null;
                                bluetoothLeService.writeCharacteristic(characteristic);
                            }
                            //bluetoothLeService.writeCharacteristic(characteristic);
                        }
                    }
                }
            }
        }
     
    }
    
    /**
     * 管理服务生命周期
     */
    private final ServiceConnection serviceConnection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName componentName, IBinder service) 
        {
        	Log.e(TAG, "管理服务生命周期on");
            bluetoothLeService = ((BluetoothLeService.LocalBinder) service).getService();
            if (!bluetoothLeService.initialize()) {
                Log.e(TAG, "无法初始化蓝牙");
            }
            if (!TextUtils.isEmpty(deviceAddress) && !TextUtils.isEmpty(deviceName)) {
            	Log.e(TAG, "初始化成功启动自动连接到设备");
                connect(deviceName, deviceAddress);
            }
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
        	Log.e(TAG, "管理服务生命周期off");
            bluetoothLeService = null;
            connected = false;
        }
    };

    /**
     * 注册广播接收者，再进行连接
     */
    public void registerReceiver(BroadcastCallback broadcastCallback) {
    	Log.e(TAG, "已经注册广播接收者");
        this.broadcastCallback = broadcastCallback;
        context.registerReceiver(broadcastReceiver, makeGattUpdateIntentFilter());
        if (bluetoothLeService != null) {
            if (!TextUtils.isEmpty(deviceAddress)) {
                final boolean result = bluetoothLeService.connect(deviceAddress);
                Log.d(TAG, "连接请求的结果：" + result);
            }
        }
    }

    private static IntentFilter makeGattUpdateIntentFilter() {
        final IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_CONNECTED);
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_DISCONNECTED);
        intentFilter.addAction(BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED);
        intentFilter.addAction(BluetoothLeService.ACTION_DATA_AVAILABLE);
        return intentFilter;
    }

    /**
     * 移除广播接收者
     */
    public void unregisterReceiver(boolean clearDevice) {
        context.unregisterReceiver(broadcastReceiver);
        broadcastCallback = null;
        if(clearDevice)
        {
        	deviceAddress = null;
        }
    }

    private boolean connected;

    /**
     * 判断是否已连接蓝牙
     *
     * @return true:已连接
     */
    public boolean isConnected() {
        return connected;
    }

    /**
     * 处理广播服务的各种活动。
     * <p/>
     * BluetoothLeService.ACTION_GATT_CONNECTED：连接到服务器GATT<p>
     * BluetoothLeService.ACTION_GATT_DISCONNECTED：从GATT服务器断开连接<p>
     * BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED：发现GATT服务<p>
     * BluetoothLeService.ACTION_DATA_AVAILABLE：从设备接收到的数据。这可以是一个读或通知
     */
    private final BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            final String action = intent.getAction();
            if (BluetoothLeService.ACTION_GATT_CONNECTED.equals(action)) {
                if (broadcastCallback != null) {             	
                    broadcastCallback.connected(intent);
                    connected = true;
                }
            } else if (BluetoothLeService.ACTION_GATT_DISCONNECTED.equals(action)) {
                connected = false;
                if (broadcastCallback != null) {
                    broadcastCallback.disconnected(intent);
                }
            } else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED.equals(action)) {
                searchGattServices(bluetoothLeService.getSupportedGattServices());
                if (broadcastCallback != null) {
                    broadcastCallback.discovered(intent);
                }
            } else if (BluetoothLeService.ACTION_DATA_AVAILABLE.equals(action)) {
                byte[] data = intent.getExtras().getByteArray(BluetoothLeService.EXTRA_DATA);
                if (broadcastCallback != null && data != null) {
                    broadcastCallback.data(intent, data);
                }
            } else {
                connected = false;
            }
        }
    };

    /**
     * 找出可用的服务列表与特征
     *
     * @param gattServices 支持的服务列表
     *                     {@code BluetoothLeService.getSupportedGattServices()}
     */
    private void searchGattServices(List<BluetoothGattService> gattServices) {
        if (gattServices == null) return;
        gattServiceData = new HashMap<BluetoothGattService,ArrayList<BluetoothGattCharacteristic>>();
        //遍历可用的服务
        Log.d(TAG, "开始遍历可用的服务");
        for (BluetoothGattService gattService : gattServices) {
        	// 根据UUID来区分大车和小车
        	Log.e(TAG,"Sevice uuid : " + gattService.getUuid().toString());
        	if (gattService.getUuid().toString().equalsIgnoreCase(UUID_SERVICE.toString()))
        	{
        		_isSmallType = 0;
        		Log.d(TAG, "isMin6 UUID_SERVICE="+gattService.getUuid().toString());
        	}
        	else if (gattService.getUuid().toString().equalsIgnoreCase(UUID_SERVICE2.toString()))
        	{
        		_isSmallType = 1;
        		Log.d(TAG, "small UUID_SERVICE2="+gattService.getUuid().toString());
        	}
        	else 
        	{
        		Log.e(TAG, "UUID_SERVICE="+gattService.getUuid().toString());
        		continue;
        	}

            ArrayList<BluetoothGattCharacteristic> gattCharacteristicList = new ArrayList<BluetoothGattCharacteristic>();
            // 遍历可用的特征
            String uuid = UUID_NOTIFY2.toString();
            if(_isSmallType == 0) uuid = UUID_NOTIFY.toString();
            for (BluetoothGattCharacteristic gattCharacteristic : gattService.getCharacteristics()) {
            	bluetoothLeService.setCharacteristicNotification(gattCharacteristic,true,_isSmallType);
                if (gattCharacteristic.getUuid().toString().equalsIgnoreCase(uuid)) {
                    gattCharacteristicList.add(gattCharacteristic);
                }
                else
                {
                	Log.d(TAG, "UUID_NOTIFY="+gattCharacteristic.getUuid().toString());
                }
            }
        
            gattServiceData.put(gattService, gattCharacteristicList);
        }
    }
    
    // true:小型车  false：大型车
    public static int IsSmallType()
    {
    	return _isSmallType;
    }
    
}
